CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: GETA
C
C*****Purpose
C     User defined routine in which the coefficient matrix
C     for the homogeneous linear differential equation is
C     defined.
C
C     NPROB = 1  (MARCUS-YAMABE, N=2)
C     NPROB = 11 (ROTATOR, N=2)
C     NPROB = 12 (REFLECTOR, N=2)
C     NPROB = 13 (ROTATOR, N=2)
C     NPROB = 14 (A=0, N ARBITRARY)
C     NPROB = 15 (A=I, N ARBITRARY)
C     NPROB = 22 (UNORDERED DIAGONAL, N=4)
C     NPROB = 2  (QUASI-PERIODIC, N=4)
C     NPROB = 3  (BROWN AND LORENZ, N=4)
C     NPROB = 4  (NEUMANN LAPLACIAN, N>=2)
C     NPROB = 5  (ROSSER, N=8)
C     NPROB = 6  (WILKINSON, N ARBITRARY)
C     NPROB = 7  (FRANK, N ARBITRARY)
C     NPROB = 100  (DIAG-LAPLACE QUASIPERIODIC, N>=2)
C     NPROB = 101  (SCALAR QUASIPERIODIC)
C     NPROB = 200  (DIECI AND LOPEZ: SYMMETRY, M=6)
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: May 29, 2003
C
CCCCCCCCCCCCCCCCCC GETA CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE GETA(M,T,AMAT,INARR,REARR)

      IMPLICIT NONE

      INTEGER M, INARR, NN
      PARAMETER (NN=4)
      INTEGER NPROB
      DOUBLE PRECISION ALP, BET, EPS
      COMMON /PARAMS/ ALP, BET, EPS, NPROB
      DOUBLE PRECISION T, REARR
      DOUBLE PRECISION AMAT(M,M)
        
      DOUBLE PRECISION QD(NN,NN), QMAT(NN,NN), AT(NN,NN)       
      DOUBLE PRECISION D2, THETT, AOFT,BOFT
      INTEGER I, J

      DO 15 I=1,M
         DO 15 J=1,M
 15         AMAT(I,J)=0.D0

      IF (NPROB.EQ.1) THEN
CCCCCCC PROBLEM MARCUS-YAMABE (N=2)
         AMAT(1,1) = -1.D0+1.5D0*DCOS(T)*DCOS(T)
         AMAT(2,2) = -1.D0+1.5D0*DSIN(T)*DSIN(T) 
         AMAT(1,2) = 1.D0-1.5D0*DSIN(T)*DCOS(T) 
         AMAT(2,1) = -1.D0-1.5D0*DSIN(T)*DCOS(T)
      END IF
      IF (NPROB.EQ.11) THEN
CCCCCCC PROBLEM ROTATOR (N=2) WITH ALP, BET
         AMAT(1,1) = BET*DCOS(2.D0*ALP*T)
         AMAT(1,2) = ALP-BET*DSIN(2.D0*ALP*T) 
         AMAT(2,1) = -ALP-BET*DSIN(2.D0*ALP*T) 
         AMAT(2,2) = -BET*DCOS(2.D0*ALP*T)
      END IF
      IF (NPROB.EQ.12) THEN
CCCCCCC PROBLEM REFLECTOR (N=2) WITH ALP, BET
         AMAT(1,1) = BET*DCOS(2.D0*ALP*T)
         AMAT(1,2) = -ALP+BET*DSIN(2.D0*ALP*T) 
         AMAT(2,1) = ALP+BET*DSIN(2.D0*ALP*T) 
         AMAT(2,2) = -BET*DCOS(2.D0*ALP*T)
      END IF
      IF (NPROB.EQ.13) THEN
CCCCCCC PROBLEM ROTATOR (N=2) WITH ALP
         THETT = ALP/(ALP*ALP+1.D0)*(EXP(-ALP*T)+ALP*DSIN(T)-DCOS(T))
         AMAT(1,1) = 0.D0
         AMAT(1,2) = ALP*(THETT-DSIN(T))
         AMAT(2,1) = -ALP*(THETT-DSIN(T))
         AMAT(2,2) = 0.D0
      END IF
      IF (NPROB.EQ.14) THEN
CCCCCCC PROBLEM A=0, ARBITRARY DIMENSIONS
         RETURN
      END IF
      IF (NPROB.EQ.15) THEN
CCCCCCC PROBLEM A=I, ARBITRARY DIMENSIONS
         DO 5 J=1,M
 5          AMAT(J,J) = 1.0D0
         RETURN
      END IF
      IF (NPROB.EQ.22) THEN
CCCCCCC DIAGONAL WITH UNORDERED DIAGONAL
         AMAT(1,1) = -1.D0/(2.D0*DSQRT(T+1.D0))
         AMAT(2,2) = -10.D0
         AMAT(3,3) = DCOS(T)
         AMAT(4,4) = 1.D0
      END IF
      IF (NPROB.EQ.2) THEN
CCCCCCC PROBLEM QP (N=4)
CCCCCCC FORM THE MATRIX (STORED IN UOUT)
         CALL BLDQ(M,T,QMAT,QD)
         DO 20 I=1,M
            DO 20 J=1,M
 20            AT(I,J) = 0.D0 
         AT(1,1) = 1.D0
         AT(2,2) = DCOS(T)
         AT(3,3) = -1.D0/(2.D0*DSQRT(T+1.D0))
         AT(4,4) = -10.D0 
         CALL BLDA(M,QMAT,QD,AT,AMAT)
      END IF
      IF (NPROB.EQ.200) THEN
CCCCCCC DIECI AND LOPEZ: SYMMMETRY
         AMAT(1,2) = 2.D0
         AMAT(1,3) = -1.D0
         AMAT(1,4) = 1.D0/(1.0D0+T)
         AMAT(1,5) = 1.D0
         AMAT(1,6) = 2.D0
         AMAT(2,1) = -2.D0
         AMAT(2,3) = 1.D0/(1.0D0+T)
         AMAT(2,4) = 5.D0
         AMAT(2,5) = DCOS(T)
         AMAT(2,6) = 4.D0
         AMAT(3,1) = 1.D0
         AMAT(3,2) = -1.D0/(1.0D0+T)
         AMAT(3,4) = 2.D0
         AMAT(3,5) = -2.0D0
         AMAT(3,6) = 1.D0
         AMAT(4,1) = -1.D0/(1.0D0+T)
         AMAT(4,2) = -5.D0
         AMAT(4,3) = -2.0D0
         AMAT(4,5) = -4.D0
         AMAT(4,6) = DCOS(T)
         AMAT(5,1) = 1.D0
         AMAT(5,2) = DCOS(T)
         AMAT(5,3) = -2.0D0
         AMAT(5,4) = -4.D0
         AMAT(5,6) = DSIN(T)
         AMAT(6,1) = 2.D0
         AMAT(6,2) = 4.D0
         AMAT(6,3) = 1.0D0
         AMAT(6,4) = DCOS(T)
         AMAT(6,5) = -DSIN(T)
      END IF
      IF (NPROB.EQ.3) THEN
CCCCCCC BROWN AND LORENZ (STIFF) (N=4)
         AMAT(1,3)=1.D0
         AMAT(2,1)=T/(2.D0*EPS)
         AMAT(2,3)=1.D0
         AMAT(2,4)=0.5D0
         AMAT(3,1)=1.D0/EPS
         AMAT(4,2)=1.D0/EPS
         AMAT(4,4)=-T/(2.D0*EPS)
      END IF
      IF (NPROB.EQ.4) THEN
CCCCCCC NEUMANN LAPLACIAN (N>=2)
	 DO 10 I=1,M
	    IF (I.EQ.1) THEN
               AMAT(1,1)=-1.D0
               AMAT(1,2)=1.D0
              ELSE IF (I.EQ.M) THEN
               AMAT(M,M)=-1.D0
               AMAT(M,M-1)=1.D0
              ELSE
               AMAT(I,I)=-2.D0
               AMAT(I,I-1)=1.D0
               AMAT(I,I+1)=1.D0
            END IF
 10	 CONTINUE
      END IF
      IF (NPROB.EQ.5) THEN
CCCCCCC ROSSER MATRIX (MUST HAVE N=8)
C HERE BELOW IS THE "ROSSER" MATRIX 
         AMAT(1,1)=611.
         AMAT(1,2)=196. 
         AMAT(1,3)=-192.  
         AMAT(1,4)=407.   
         AMAT(1,5)=-8.  
         AMAT(1,6)=-52.  
         AMAT(1,7)=-49.   
         AMAT(1,8)=29.      
         AMAT(2,1)=196.  
         AMAT(2,2)=899.  
         AMAT(2,3)=113. 
         AMAT(2,4)=-192.  
         AMAT(2,5)=-71.  
         AMAT(2,6)=-43.   
         AMAT(2,7)=-8.  
         AMAT(2,8)=-44.      
         AMAT(3,1)=-192.  
         AMAT(3,2)=113.  
         AMAT(3,3)=899.  
         AMAT(3,4)=196.   
         AMAT(3,5)=61.   
         AMAT(3,6)=49.    
         AMAT(3,7)=8.   
         AMAT(3,8)=52.      
         AMAT(4,1)=407. 
         AMAT(4,2)=-192.  
         AMAT(4,3)=196.  
         AMAT(4,4)=611.    
         AMAT(4,5)=8.   
         AMAT(4,6)=44.   
         AMAT(4,7)=59.  
         AMAT(4,8)=-23.      
         AMAT(5,1)=-8.  
         AMAT(5,2)=-71.   
         AMAT(5,3)=61.    
         AMAT(5,4)=8.  
         AMAT(5,5)=411. 
         AMAT(5,6)=-599.  
         AMAT(5,7)=208.  
         AMAT(5,8)=208.      
         AMAT(6,1)=-52.  
         AMAT(6,2)=-43.   
         AMAT(6,3)=49.   
         AMAT(6,4)=44. 
         AMAT(6,5)=-599.  
         AMAT(6,6)=411.  
         AMAT(6,7)=208.  
         AMAT(6,8)=208.  
         AMAT(7,1)=-49.
         AMAT(7,2)=-8.    
         AMAT(7,3)=8.   
         AMAT(7,4)=59.  
         AMAT(7,5)=208.  
         AMAT(7,6)=208.   
         AMAT(7,7)=99. 
         AMAT(7,8)=-911.      
         AMAT(8,1)=29.  
         AMAT(8,2)=-44.   
         AMAT(8,3)=52.  
         AMAT(8,4)=-23.  
         AMAT(8,5)=208.  
         AMAT(8,6)=208. 
         AMAT(8,7)=-911.   
         AMAT(8,8)=99.
      END IF       
      IF (NPROB.EQ.6) THEN
CCCCCCC WILKINSON MATRIX (N IS VARIABLE)
C DEFINE "WILKINSON" MATRIX OF DIMENSION N 
         DO 11 I=1,M-1
            AMAT(I,I+1)=1.D0
 11         AMAT(I+1,I)=1.D0
         D2=(M-1)/2.D0
         DO 14 I=1,M
 14         AMAT(I,I)=DABS(D2-I+1)
      END IF
      IF (NPROB.EQ.7) THEN
CCCCCCC FRANK MATRIX (N IS VARIABLE)
C NEXT IS THE "FRANK" MATRIX
         DO 29 I=1,M
            DO 29 J=I,M
 29            AMAT(I,J)=M-J+1
         DO 32 J=2,M
 32         AMAT(J,J-1)=AMAT(J,J) 
      END IF
      IF (NPROB.EQ.100) THEN
CCCCCCC DIAG-LAPLACIAN QUASI-PERIODIC (N>=2)
         D2 = 1.D0/DBLE(M-1)
         D2=1.D0/(D2*D2)
         AOFT = (2.D0+COS(LOG(1.D0*T+1.D0))+SIN(LOG(1.D0*T+1.D0)))*D2
         BOFT = COS(LOG(1.D1*T+1.D0))+SIN(LOG(1.D1*T+1.D0))
C         BOFT = COS(LOG(1.D0*T+1.D0))+SIN(LOG(1.D0*T+1.D0))
C         BOFT = 0.0D0
         DO 40 I=1,M
	    IF (I.EQ.1) THEN
               AMAT(1,1)=-1.D0*AOFT+BOFT
               AMAT(1,2)=1.D0*AOFT
              ELSE IF (I.EQ.M) THEN
               AMAT(M,M)=-1.D0*AOFT+BOFT
               AMAT(M,M-1)=1.D0*AOFT
              ELSE
               AMAT(I,I)=-2.D0*AOFT+BOFT
               AMAT(I,I-1)=1.D0*AOFT
               AMAT(I,I+1)=1.D0*AOFT
            END IF
 40      CONTINUE
      END IF
      IF (NPROB.EQ.101) THEN
CCCCCCC SCALAR QUASI-PERIODIC 
         AOFT = COS(LOG(1.D0*T+1.D0))+SIN(LOG(1.D0*T+1.D0))
         BOFT = COS(LOG(1.D1*T+1.D0))+SIN(LOG(1.D1*T+1.D0))
         BOFT = 0.0D0
         AMAT(1,1)=AOFT+BOFT
      END IF

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: BLDQ 
C
C*****Purpose
C     Used to form an orthgonal matrix as part of 
C     the example problem nprob=2
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: August 2000
C
CCCCCCC ROUTINE BLDQ
      SUBROUTINE BLDQ(N,T,Q,QD)

      IMPLICIT NONE

      INTEGER N
      DOUBLE PRECISION T
      DOUBLE PRECISION Q(N,N), QD(N,N)

      INTEGER I,J
      DOUBLE PRECISION DRT2

      DRT2 = DSQRT(2.D0)
      DO 10 J=1,N
         DO 10 I=1,N
            QD(I,J) = 0.D0
 10         Q(I,J) = 0.D0
      Q(1,1) = DCOS(T)
      Q(1,2) = DSIN(T)
      Q(2,1) = -DSIN(T)*DCOS(DRT2*T)
      Q(2,2) = DCOS(T)*DCOS(DRT2*T)
      Q(2,3) = DCOS(T)*DSIN(DRT2*T)
      Q(2,4) = DSIN(T)*DSIN(DRT2*T)
      Q(3,1) = DSIN(T)*DSIN(DRT2*T)
      Q(3,2) = -DCOS(T)*DSIN(DRT2*T)
      Q(3,3) = DCOS(T)*DCOS(DRT2*T)
      Q(3,4) = DSIN(T)*DCOS(DRT2*T)
      Q(4,3) = -DSIN(T)
      Q(4,4) = DCOS(T)
      QD(1,1) = -DSIN(T)
      QD(1,2) = DCOS(T)
      QD(2,1) = -DCOS(T)*DCOS(DRT2*T)
     *          + DRT2*DSIN(T)*DSIN(DRT2*T)
      QD(2,2) = -DSIN(T)*DCOS(DRT2*T)
     *          - DRT2*DCOS(T)*DSIN(DRT2*T)
      QD(2,3) = -DSIN(T)*DSIN(DRT2*T)
     *          + DRT2*DCOS(T)*DCOS(DRT2*T)
      QD(2,4) = DCOS(T)*DSIN(DRT2*T)
     *          + DRT2*DSIN(T)*DCOS(DRT2*T)
      QD(3,1) = DCOS(T)*DSIN(DRT2*T)
     *          + DRT2*DSIN(T)*DCOS(DRT2*T)
      QD(3,2) = DSIN(T)*DSIN(DRT2*T)
     *          - DRT2*DCOS(T)*DCOS(DRT2*T)
      QD(3,3) = -DSIN(T)*DCOS(DRT2*T)
     *          - DRT2*DCOS(T)*DSIN(DRT2*T)
      QD(3,4) = DCOS(T)*DCOS(DRT2*T)
     *          - DRT2*DSIN(T)*DSIN(DRT2*T)
      QD(4,3) = -DCOS(T)
      QD(4,4) = -DSIN(T)

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: BLDA 
C
C*****Purpose
C     Forms the coefficient matrix for the example
C     problem with nprob=2
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: August 2000
C
CCCCCCCC ROUTINE BLDA
      SUBROUTINE BLDA(M,QMAT,QD,AT,A)

      IMPLICIT NONE

      INTEGER M
      INTEGER NN
      PARAMETER (NN=4)
      INTEGER NPROB
      DOUBLE PRECISION ALP, BET, EPS
      COMMON /PARAMS/ ALP, BET, EPS, NPROB
      DOUBLE PRECISION QMAT(M,M), QD(M,M), A(M,M), AT(M,M)

      DOUBLE PRECISION QTMP1(NN,NN), QTMP2(NN,NN)
      INTEGER I, J

      CALL MATTMT(QMAT,M,M,M,AT,A)
      CALL MATTMH(A,M,M,M,QMAT,QTMP1)
      CALL MATTMH(QD,M,M,M,QMAT,QTMP2)
      DO 10 J=1,M
         DO 10 I=1,M
 10         A(I,J) = QTMP1(I,J) + QTMP2(I,J)

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: GETQ 
C
C*****Purpose
C     Forms an orthogonal matrix as the product of two
C     orthogonal matrices for the example nprob=2
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: August 2000
C
CCCCCCC GETQ CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE GETQ(M,NQ1,T,QMAT)

      IMPLICIT NONE

      INTEGER M,NQ1
      DOUBLE PRECISION T, QMAT(M,NQ1)
      INTEGER NN
      PARAMETER (NN=4)

      INTEGER NPROB
      DOUBLE PRECISION ALP, BET, EPS
      COMMON /PARAMS/ ALP, BET, EPS, NPROB
      
      INTEGER I,J
      DOUBLE PRECISION QMAT1(NN,NN), QMAT2(NN,NN)
      DOUBLE PRECISION QMAT4(NN,NN)

      CALL BLDQ(NN,T,QMAT1,QMAT2)
      DO 10 I=1,M
         DO 10 J=1,NQ1
            QMAT4(I,J)=0.D0
 10         IF (I.EQ.J) QMAT4(I,J)=1.D0 
      CALL MATTMT(QMAT1,M,M,NQ1,QMAT4,QMAT)

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: ORTCHK
C
C*****Purpose
C     Sample testing routine to determine the error from
C     orthogonality and --for problems where the exact
C     solution is known-- the global error.
C
C*****Authors: L Dieci & ES Van Vleck 
C*****Date: October 2001
C
CCCCCCC ORTCHK CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE ORTCHK(M,NQ1,T,QMAT,ATILDE,ALPHA,BETA,ERRA,ERRD,
     *                        RMAT,QEXAC)

      IMPLICIT NONE

      INTEGER NPROB
      DOUBLE PRECISION ALP, BET, EPS
      COMMON /PARAMS/ ALP, BET, EPS, NPROB
      INTEGER M,NQ1
      DOUBLE PRECISION T,QMAT(M,NQ1), ALPHA, BETA
      DOUBLE PRECISION ATILDE(M,M), ERRA, ERRD

      INTEGER I,J,NUMREF
      DOUBLE PRECISION RMAT(NQ1,NQ1), QEXAC(M,NQ1)
      DOUBLE PRECISION THETT

      NUMREF=MIN(NQ1,M-1)
      DO 40 I=1,M
         DO 40 J=1,NQ1
 40         QEXAC(I,J)=0.D0
      CALL MATHTM(QMAT,NQ1,M,NQ1,QMAT,RMAT)
      IF (NPROB.EQ.2) THEN
         CALL GETQ(M,NQ1,T,QEXAC)
        ELSE IF (NPROB.EQ.12) THEN
         QEXAC(1,1)=DCOS(ALP*T)
         QEXAC(1,2)=-DSIN(ALP*T)
         QEXAC(2,1)=DSIN(ALP*T)
         QEXAC(2,2)=DCOS(ALP*T)
        ELSE IF (NPROB.EQ.13) THEN
         THETT=ALP/(ALP*ALP+1.D0)*(EXP(-ALP*T)+ALP*DSIN(T)-DCOS(T))
         QEXAC(1,1)=DCOS(THETT)
         QEXAC(1,2)=-DSIN(THETT)
         QEXAC(2,1)=DSIN(THETT)
         QEXAC(2,2)=DCOS(THETT)
      END IF
      ALPHA=0.D0
      BETA=0.D0
      DO 10 I=1,NQ1
         DO 20 J=1,NQ1
            IF (I.EQ.J) RMAT(I,J)=RMAT(I,J)-1.D0
            ALPHA=DMAX1(ALPHA,DABS(RMAT(I,J)))
 20      CONTINUE
         DO 30 J=1,M
            IF ((NPROB.EQ.12).OR.
     *         (NPROB.EQ.13).OR.(NPROB.EQ.2)) THEN
               BETA=DMAX1(BETA,DABS(QMAT(J,I)-QEXAC(J,I)))
            END IF
 30      CONTINUE
 10   CONTINUE
      ERRA=0.D0
      ERRD=0.D0
C      DO 50 I=1,NUMREF
C         DO 50 J=I+1,M
      DO 50 I=1,NQ1
         DO 50 J=I+1,NQ1
            ERRD=DMAX1(ERRD,DABS(ATILDE(I,J)))
 50   ERRA=DMAX1(ERRA,DABS(ATILDE(J,I)))
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: MATTMT 
C
C*****Purpose
C     Forms the matrix product of two matrices.
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: August 2000
C
CCCCCCCCCCCCCCCCCC MATTMT CCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE MATTMT(A,N1,N2,N3,B,C)

      IMPLICIT NONE

      INTEGER N1,N2,N3
      DOUBLE PRECISION A(N1,N2), B(N2,N3), C(N1,N3)
      INTEGER I,J,K

      DO 10 J=1,N3
         DO 10 I=1,N1
            C(I,J) = 0.D0
            DO 10 K=1,N2
 10            C(I,J) = C(I,J) + A(I,K)*B(K,J)

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: MATHTM
C
C*****Purpose
C     Forms the matrix product A^T B.
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: August 2000
C
CCCCCCCCCCCCCCCCCC MATHTM CCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE MATHTM(A,N1,N2,N3,B,C)

      IMPLICIT NONE

      INTEGER N1,N2,N3
      DOUBLE PRECISION A(N2,N1), B(N2,N3), C(N1,N3)
      INTEGER I,J,K

      DO 10 J=1,N3
         DO 10 I=1,N1
            C(I,J) = 0.D0
            DO 10 K=1,N2
 10            C(I,J) = C(I,J) + A(K,I)*B(K,J)

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C*****Subroutine name: MATTMH
C
C*****Purpose
C     Forms the matrix product A B^T.
C
C*****Authors: L Dieci & ES Van Vleck
C*****Date: August 2000
C
CCCCCCCCCCCCCCCCCC MATTMH CCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE MATTMH(A,N1,N2,N3,B,C)

      IMPLICIT NONE

      INTEGER N1,N2,N3
      DOUBLE PRECISION A(N1,N2), B(N3,N2), C(N1,N3)
      INTEGER I,J,K

      DO 10 J=1,N3
         DO 10 I=1,N1
            C(I,J) = 0.D0
            DO 10 K=1,N2
 10            C(I,J) = C(I,J) + A(I,K)*B(J,K)

      RETURN
      END

